"""
This is the docstring for the examplesubpkg package.  Normally you would
have whatever.py files in this directory implementing some modules, but this
is just an example sub-package, so it doesn't actually do anything.
"""
__all__ = []

from .coreClass import *
from .readerClass import *
from .utils import *
from .cacheClass import *

__all__ += coreClass.__all__
__all__ += readerClass.__all__
__all__ += cacheClass.__all__
__all__ += utils.__all__


from astroquery.astrometry_net import AstrometryNet
from astroquery.exceptions  import TimeoutError
from astropy.wcs import WCS

ast = AstrometryNet()

__all__ =  ['plate_solve', 'get_night', 'getDateString', 'CCDDatax']

from astropy.nddata.ccddata import CCDData
CCDData._config_ccd_requires_unit = False
import astropy.units as un

CCDDatax = CCDData
CCDDatax.unit = un.adu # this is what the kids call hardcoding and I'm doing it because astropy refuses to fix their godforsaken unit issue with CCDData
# why did CCDData decide to randomly break? who knows
# why did CCDData start refusing to achknowledge me telling it explicitely multiple times that it doesn't need a unit
# because when I hand it a unit it ignores it and instead just reads it from the header? I don't know, does astropy team know?
# probably not. But its a thing and I really wish it wasn't at this hour of the night.


def plate_solve(dirarray, data = None, writearray = False):
    """
    plate_solve takes a dirarray pointing to a fits file to plate solve using astrometryNet. It returns
    the solved image with the WCS header and the WCS header itself if the solve succeeded. If 'writearray'
    is set with a filepath array, a copy of the solved image will be saved.
    
    Parameters
    ----------
    dirarray: str array
        filepath array to image to be solved.
    data: CCDData
        optional image data to combine with the WCS header on return.save. Default is data to be solved.
    Returns
    -------
    
    """
    # TODO better handle cache from here and check if API key
    # can we print a summary of the solve?
    # path = Dorado.dordir
    from pathlib import Path
    path = Path()
    for dir in dirarray:
        path = path / dir

    if data == None:
        data = CCDDatax.read(path) #, unit = Dorado.unit) ## NOTE edited

    trying = True
    submission_id = None
    num = 0

    while trying:
            try:
                if not submission_id:
                    wcs_header = ast.solve_from_image(path, force_image_upload=True, submission_id=submission_id, solve_timeout=300)
                else:
                    print('Monitoring: try #', num)
                    wcs_header = ast.monitor_submission(submission_id, solve_timeout=300)
            except TimeoutError as e:
                print(TimeoutError)
                num = num + 1
                print('Timed out: try #', num)
                submission_id = e.args[1]

            if wcs_header != None:
                # got a result, so terminate while loop
                trying = False
    if wcs_header:
        # Code to execute when solve succeeds
        print('Solve succeeded! :)')
        wcs_hdu = data
        wcs_hdu.header = wcs_header
        if writearray:
            path = Path()
            for dir in writearray:
                path = path / dir
            wcs_hdu.write(path, overwrite = True)

        return wcs_hdu, wcs_header
    else:
        # Code to execute when solve fails
        print('Solve failed! :(')
        return 


import datetime

def get_night():
    """
    get_night obtains a timestring for the most recent(previous) night based on local/hardware
    time provided by datetime. The format follows yyyy-mm-(dd-1)+dd where (dd-1) is last nights 
    day of the month. This currently does not support dates at which the start of the observing
    night was the last day of the month.

    Parameters
    ----------
    None

    Returns
    -------
    night: str
            Timestring for the most recent night.
    """
    # currently does not support first/last of the month
    # option for last night or tonight
    # UTC or local time?
    date = datetime.date.today()
    year = date.year
    month = date.month
    day = date.day
    # date1 = date2 - 1
    # night = str(year) + '-' + str(month) + '-' + str(date1) + '+' + str(date2)


    if day < 10:
        daystr = '0' + str(day)
    else:
        daystr = str(day)
    if day < 9:
        day2str = '0' + str(day + 1)
    else:
        day2str = str(day + 1)
    if month < 10:
        monthstr = '0' + str(month)
    else:
        monthstr = str(month)


    night = str(year) + '-' + monthstr + '-' + daystr + '+' + day2str




    return night
        
from astropy.time import Time

def getDateString(epoch, utc = None):
    """
    getDateString computes the date string for the given ceres object. ceres time is represented
    in UTC while datestring is represented in local time. A UTC offset must be provided either by self.UTCoffest
    or by the utc arguement.
    
    Parameters
    ----------
    cr : str
        the ceres object key to get the date string for.
    utc: float
        utc offset to apply to the date string. Default is None.

    Returns
    -------
    
    """    
    if type(epoch) == str:
        epoch = Time(epoch, format='fits')
    elif type(epoch) != type(Time):
        raise Exception("Could not parse input")
    # epoch = date.ymdhms <- desired time instance format
    # Fall back to Dorado UTC offset if UTC offset is not provided
    if not (utc):
        utc = Dorado.UTCoffset
    epdt = epoch.datetime
    # if the hour is less than the utc offset of the site then the utc date is one ahead of local time
    if (epdt.hour + utc) < 0:
        day = str(epdt.day - 1)
        day2 = str(epdt.day)
        month = str(epdt.month)
        if (epdt.day - 1) < 10:
            day = '0' + str(epdt.day - 1)
        if epdt.day < 10:
            day2 = '0' + str(epdt.day)
        if epdt.month < 10:
            month = '0' + str(epdt.month)

    else: 
        day = str(epdt.day)
        day2 = str(epdt.day + 1)
        month = str(epdt.month)
        if epdt.day < 10:
            day = '0' + str(epdt.day)
        if epdt.day < 9:
            day2 = '0' + str(epdt.day + 1)
        if epdt.month < 10:
            month = '0' + str(epdt.month)

    return str(epdt.year) + '-' + month + '-' + day + '+' + day2
    


import warnings
warnings.filterwarnings('ignore')

from ..ceres import Ceres
from ..stack import Stack
from ..core.coreClass import *
from ..core.utils import *

# import utils
import ccdprocx

# from astropy.nddata.ccddata import CCDData
# CCDData._config_ccd_requires_unit = False
import astropy.units as un
from astropy.time import Time
from astropy.io import fits
from astropy.utils.misc import isiterable

import lightkurve as lk
from lightkurve.lightcurve import TessLightCurve as tlc

import os
from tqdm import tqdm
import numpy as np

__all__ = ['reader', 'aico_reader'] #, 'tess_reader'

class reader:
    '''
        readers need to have a dirscan method and a savewrok method. Readers should also probably have a mkceres method.
    '''
    def __init__(self):
        # should report to logger and read from config here
        # will this carry over to inherited classes?
        self.desired_datatype = 'uint16' # datatype to force onto images

    def diread(self, dirarray):
        """
        diread intakes a filepath array and catalogues the contents by either 'is file' or 'is directory'
        and returns the resulting lists.
        
        Parameters
        ----------
        dirarray: str array
            array containing the path to the desired directory.
        Returns
        -------
        
        """
        if isiterable(dirarray):
            path     = Dorado.dordir
            for dir in dirarray:
                path = path / dir
        else:
            path     = dirarray
        # path       = self.dordir / 'data' / 'raw' / date
        contents     = os.scandir(path = path)
        files        = []
        directories  = []
        for entry in contents:
            if not entry.name.startswith('.'):
                if entry.is_file():
                    files.append(entry)
                if entry.is_dir():
                    directories.append(entry)
        return files, directories

    def newdat(self, only_new = True, handback = True, print_out = True):
        """
        newdat is an unfinished function that will scan the '$user/.dorado/data/raw' directory
        for new data to be processed.
        
        Parameters
        ----------

        Returns
        -------
        
        """
        print('\n|  Data Scan  |')
        # scan data directories
        _, raw_directories = self.diread(['data', 'raw'])
        _, wrk_directories = self.diread(['data', 'wrk'])
        datdirs = []
        if only_new:
            for d in raw_directories:
                if d.name not in [w.name for w in wrk_directories]:
                    datdirs.append(d)
        else:
            datdirs = raw_directories
        if print_out:
            print('Data directories found: \n')
            for d in datdirs:
                print('-----| ' + d.name + ' \n')
        if handback:
            return datdirs

    def force16(self, hdu):
        """
        An unfinished convinience function for forcing the datatype of CCDData to be 16-bit instead of 32-bit
        for consistency and filesize optimization.
        
        Parameters
        ----------
        hdu: CCDData
            hdu to force 16-bit datatype.
        Returns
        -------
        
        NOTE:: in the future this will handle arrays of data instead of single HDU's and will allow for specifying the bit datatype.
        """
        print('This function is not implemented yet. See mkBias() for example functionality.')




# TODO Make it so that each class isnt loaded unless chosen (core class takes string and then imports correct class)
class aico_reader(reader):
    '''
    
    '''
    def __init__(self):
        self.biasstr            = ['Bias', 'bias', 'BIAS']
        self.flatsstr           = ['FLAT', 'FlatField', 'flat', 'Flat', 'Flats', 'flats', 'FLATS', 'FlatFields']
        self.lightsstr          = ['lights', 'Lights', 'LIGHTS']
        self.stack_types        = {'bias':self.biasstr, 'flats':self.flatsstr, 'lights':self.lightsstr}
        self.unique_lights      = True # whether to find lights by non calibration files (True) or via self.lightstr (False)
        self.require_cal        = False  # whether calibration frames are essential, this should be dynamically set by the read calibration level
        self.calibration_window = 7

    def _single_level(self, files):
        bias    = self._read_stack(files, 'bias'  )
        flats   = self._read_stack(files, 'flats' )
        lights  = self._read_stack(files, 'lights')
        return bias, flats, lights

    def _multi_level_top(self, files, directories):
        biasdir = [s for s in directories if s.name in self.biasstr]
        # There should never be multiple bias directories; unless bias were taken with multiple binning settings or taken multiple times.
        # Consult chief data officer about this. TODO
        if  len(biasdir)  > 0:
            bias_files, _ = self.diread(biasdir[0])
            bias          = self._read_stack(bias_files, 'bias')
        else:
            bias          = []
        # It is possible to have multiple flats directories for different filters, they should all be contained in a single base flats directory tho
        flatsdir          = [s for s in directories if s.name in self.flatsstr]
        if len(flatsdir) > 0:
            flats         = self._multi_level_sub(flatsdir[0], 'flats')
        else:
            flats         = []
        if self.unique_lights:
            lightsdir     = [s for s in directories if (s.name not in self.flatsstr) and (s.name not in self.biasstr)] # may hand back more than one directory
        else:
            lightsdir     = [s for s in directories if s.name in self.lightsstr]  # may hand back more than one directory
        if len(lightsdir) > 0: # should also handle more than one rogue dir
            lights        = self._multi_level_sub(lightsdir[0], 'lights')
        else:
            lights        = []
        return bias, flats, lights

    def _multi_level_sub(self, subdir, stack_type):
        '''
        File doubling comes from mix directory type, and the fact
        that every type (apparently not lights tho) is regarded as 
        a mix directory due to non-stack_type directories and files 
        being present in the diread.
        '''
        files, directories   = self.diread(subdir)
        sub_files = []
        if len(directories) == 0 & len(files) > 0:
            print('Single directory ', stack_type, ' organization format detected.')
            sub_files        = self._read_stack(files, stack_type)
        elif len(files) == 0 & len(directories) > 0:
            print('Multi directory ', stack_type, ' organization format detected.')
            for d in directories:
                # should not be any further sub folders
                f, _         = self.diread(d)
                sub_files   += self._read_stack(f, stack_type) # should we continue to hand stack_type if only the desired frame type should be present this deep in the mix?
        elif len(files) == 0 & len(directories) == 0:
            print('No', stack_type, '  detected.')
        else:
            print('Mix directory ', stack_type, ' organization format detected.')
            sub_files        = self._read_stack(files, stack_type)
            for d in directories:
                # should not be any further sub folders
                print(d)
                f, _         = self.diread(d)
                # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= #
                #
                # Check if previous and current list is same to prevent file doubling
                #
                # =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-= #
                sub_files   += self._read_stack(f, stack_type)
        return sub_files

    def _read_stack(self, files, stack_type):
        if (stack_type == 'lights') & (self.unique_lights):
            stackstr     = self.stack_types['bias'] + self.stack_types['flats'] # NOTE:: in the future this should be modded to include all non light stack types.
        else:
            try:
                stackstr = self.stack_types[stack_type]
            except:
                stackstr = stack_type
            
        stack_list = []
        for s in files:
            good = True # janky way of making sure we dont double up when a few str dont match
            for st in stackstr:
                if (stack_type == 'lights') & (self.unique_lights):
                    if (str(st)) in str(s.name):
                        good = False
                        
                else:
                    if (str(st)) in str(s.name):
                        stack_list.append(s)
            if (stack_type == 'lights') & (self.unique_lights):
                if good:
                    stack_list.append(s)
        stack_files = []
        if len(stack_list) > 50:
            n = 0 # this is a very janky way of writing this, enjoy
            for f in (pbar := tqdm(stack_list, colour = 'green')):
                n += 1
                pbar.set_description('Reading ' + stack_type + ' : ' + str(n))
                pbar.refresh()
                hdu = CCDDatax.read(f.path)
                stack_files.append(hdu)
        elif len(stack_list) > 0:
            for f in stack_list:
                hdu = CCDDatax.read(f.path)
                stack_files.append(hdu)
        else:
            raise Exception('Error locating files: ', stack_type, ' not found.')
        return stack_files

    def dirscan(self, dirarray):
        '''
        Dirscan takes a filepath array and scans the resulting directory for Bias, Flats, and Lights
        based on a simple filename pattern. Data can be either all in this single directory or in appropriately 
        named sub directories(i.e. '/bias', '/flats', and '/lights'). 
        
        **This currently only supports single filter data** 
        
        Parameters
        ----------
        dirarray: str array
            array containing the path to the desired directory.
        '''
        # TODO handle multi filter searching 
        path = Dorado.dordir
        for dir in dirarray:
            path = path / dir
        files, directories = self.diread(path)

        # TODO check if files are actually fits files before we break the fits reader
        if len(directories) == 0:
            if len(files) == 0:
                raise Exception('No viable data found')

            else:
                print('Single directory level organization format detected.')
                return self._single_level(files)


        elif len(files) == 0:
            print('Multi directory level organization format detected.')
            # People need to adopt a standard way of saving stuff or I will go nuts trying to come up with all the different ways they can organize(or disorganize) their data.
            return self._multi_level_top(files, directories) # what if there is usable data in single level and we only search multi-level

    def _chkPhi(self, stack_files):
        # sequence through headers saving every filter keyword
        phi_list = []
        phi_stacks = {}
        for im in stack_files:
            phi = im.header['FILTER'] # hardcoded filter keyword for now
            if phi in phi_list: # check if filter is already accounted for 
                phi_stacks[phi].append(im)
            else:
                phi_list.append(phi)
                phi_stacks[phi] = [im]
        for p in phi_list:
            print('Found ', len(phi_stacks[p]), ' ', p, ' frames.')
        # Should we go ahead and construct stacks first before returning?
        return phi_list, phi_stacks

    def _chkCal(self):
        pass

    def mkceres(self,  date, sub = 'raw', target = None, calibrated = False, aligned = False):
            '''
            mkceres creates a ceres object from a given datestring for an observation and an optional 
            dorado.target instance. mkceres can be pointed to calibrated or aligned data via the
            calibrated and aligned boolean flags (Note, in this case 'sub' should be set to 'wrk').
            
            Parameters
            ----------
            date: date string
                Instance of dorado.stack class to add to self.
            sub: string
                pointer string denoting subfolder to '$user/.dorado/data' (dorado directory/data/).
                Default is 'raw'
            target: dorado.target
                dorado.target instance to use as the target for the created dorado.ceres
                object. Default is None

            calibrated: boolean
                sets whether the imported images are calibrated and within an 'calibrated'
                folder. Default is False.

            aligned: boolean
                sets whether the imported images are aligned and within an 'aligned'
                folder. Default is False.
            '''
            print('\n|  Reading  |')
            # TODO needs ability to handle multifilter directories
            # TODO auto handle setting 'wrk' as subfolder
            # TODO ignore looking for calibration files if calibrated or aligned 
            if aligned:
                dirarray = ['data', sub, date, 'aligned']
            elif calibrated:
                dirarray = ['data', sub, date, 'calibrated']
            else:
                dirarray = ['data', sub, date]

            biases, flats, lights = self.dirscan(dirarray)

            print(len(flats),  ' flats found.')
            print(len(biases), ' bias frames found.')
            print(len(lights), ' lights found')
            
            # lets split the flats into filters
            flat = {}
            if len(flats) > 0:
                phi_flat_list, phi_flat_stacks = self._chkPhi(flats)
                print('Flat filters found : ', phi_flat_list)
                for f in phi_flat_list:
                    print('Making ', f, ' flat.')
                    flat[f] = self.mkFlat(phi_flat_stacks[f])
            else:
                phi_flat_list = [] # set empty array to check if lights filters are found in flat filters.
            # Make the  Base Bias if possile
            if len(biases) > 0:
                bias = self.mkBias(biases)
                
            if len(lights) > 0:
                epoch = lights[0].header['DATE-OBS']
                datestr = getDateString(epoch) # TODO 
                mjdstr  = str(int(np.round(Time(epoch, format='fits').mjd, 0)))
            else:
                raise Exception("No usable lights found") # NOTE user may want to run calibration files without lights, or have lights without calibration files

            # Get Base Bias if none in raw
            if len(biases) == 0:
                bias = self.getBias(mjdstr) # find bias that are near in time
                
            # lets split the lights into filters
            phi_list, phi_stacks = self._chkPhi(lights)
            print('Light filters found : ', phi_list)
        
            for phi in phi_list: # This is not a very stable section, needs some love later to handle niche cases
                # find missing flats near in time to data
                if phi not in phi_flat_list:
                    # print('Retrieving ', phi, ' flat.')
                    phi_flat = self.getFlat(mjdstr, phi)
                    flat[phi] = phi_flat
            
            # Construct cere class
            cere = Ceres(time = Time(epoch, format='fits'), datestr=datestr)
            cere.bias = bias
                    
            # Construct stacks and hand them to Ceres
            for phi in phi_list:
                cere.add_stack(Stack(phi_stacks[phi], flat = flat[phi],  calibrated = calibrated, aligned = aligned, target = target))
                
            print('\n')
            return cere

    def mkFlat(self, flats):
            """
            mkFlat takes  a list of flats to construct a calibrated flatfield image.
            
            Parameters
            ----------
            flats: array[CCDdata]
                    array of raw flatfields. ------------> this needs to be corrected for the new image storage format

            Returns
            -------
            flat: CCDdata
                    The combined calibrated flatfield image.
            """
            date = Time(flats[0].header['DATE-OBS'], format='fits').mjd
            filt = flats[0].header['filter']
            ## TODO :: standardize filter names and include telescope profiles
            fname = str(int(date)) + '_' + str(filt) + '_flat.fits'
            flatdir = Dorado.dordir / 'data' / 'flats' 
            contents = os.scandir(path = flatdir)
            save = True
            for entry in contents:
                if fname in entry.name:
                    save = False
                    flat = CCDDatax.read(flatdir / fname) #, unit = Dorado.unit) ## NOTE edited
            # TODO Allow RGB data
            
            
            if save:
                import timeit
                start = timeit.default_timer()
                # from datetime import datetime
                # start=datetime.now()
                flat = ccdprocx.combine(flats, method='median', mem_limit=2e9, sigma_clip=True)
                stop = timeit.default_timer()
                print('Time to stack ', filt, ' flats: ', stop - start)
                # print datetime.now()-start
                # c = ccdprocx.Combiner(flats)
                # c.sigma_clipping()
                # flat = c.median_combine()
                # , method = 'average',
                #                     sigma_clip = True, sigma_clip_low_thresh = 5, sigma_clip_high_thresh = 5,
                #                     sigma_clip_func = np.ma.median, sigma_clip_dev_func = mad_std, unit = self.unit)
                flat.header['stacked'] = True
                flat.header['numsubs'] = len(flats)
                flat.header['DATE-OBS'] = flats[0].header['DATE-OBS']
                flat.header['filter'] = flats[0].header['filter']
                ## TODO :: There is probably more missing keywords in the combined header, where's Waldo...

                
                flat.data = flat.data.astype('uint16') 

                print('Saving Flat for later use')
                flat.write(flatdir / fname)
            else:
                print('Flat for filter and date already saved.')

            return flat

    def mkBias(self, biasIFC):
            """
            mkBias takes a list of bias images to construct 
            a combined bias image. ------------> this needs to be corrected for the new image storage format
            
            Parameters
            ----------
            biasIFC: array[CCDdata]
                    array of raw bias images.

            Returns
            -------
            bias: CCDdata
                    The combined bias image.
            """
            # Allow specification of median or mean
            # Allow RGB data

            date = Time(biasIFC[0].header['DATE-OBS'], format='fits').mjd
            fname = str(int(date)) + '_Bias.fits'
            biasdir = Dorado.dordir / 'data' / 'bias' 
            contents = os.scandir(path = biasdir)
            save = True
            for entry in contents:
                if fname in entry.name:
                    save = False
                    bias = CCDDatax.read(biasdir / fname) #, unit = Dorado.unit) ## NOTE edited
            if save:
                bias = ccdprocx.combine(biasIFC, method = 'average', mem_limit=2e9) #, unit = Dorado.unit) ## NOTE edited
                bias.meta['stacked'] = True
                bias.header['numsubs'] = len(biasIFC)
                # date = Time(bias.header['DATE-OBS'], format='fits').mjd
                bias.data = bias.data.astype('uint16')
                print('Saving Bias for later use')
                bias.write(biasdir / fname)
            else:
                print('Bias for date already saved.')
            return bias

    def savewrk(self, cr, filters = None, save_series = True, save_base = True):
        """
        savewrk takes a ceres key and a list of filters to save and saves the desired data
        to the dorado working directory '$user/.dorado/data/wrk/'. 
        
        Parameters
        ----------
        cr: string
            The relevent name string of Ceres instance to save.
        filter: str
            String representation of the relevent filter. Default is all filters in ceres instance.

        Returns
        -------
        
        """
        print('\n|  Saving  |')
        # TODO mod fplate to accept cr name
        if Dorado.ceres[Dorado.ceres_keys[cr]].datestr == None:
            Dorado.getDateString(cr) 
        # wrkdir = Dorado.dordir / 'data' / 'wrk'

        datestr = Dorado.ceres[Dorado.ceres_keys[cr]].datestr
        print('Save to data/wrk/', datestr)
        Dorado.mkwrk(cr)

        if filters == None:
            filters = Dorado.ceres[Dorado.ceres_keys[cr]].filters.keys()
        
        if type(filters) == type([]):
            for phi in filters:
                self._save_phi(cr, phi, save_series, save_base)
        elif type(filters) == str:
            self._save_phi(cr, filters, save_series, save_base)
        else:
            raise Exception('Cannot parse filter arguement. Did you enter the string of a single filter or a list of filter strings?')
        print('\n')

    def _save_phi(self, cr, phi, save_series, save_base):
        wrkdir  = Dorado.dordir / 'data' / 'wrk'
        datestr = Dorado.ceres[Dorado.ceres_keys[cr]].datestr
        fildat  = Dorado.ceres[Dorado.ceres_keys[cr]].data[Dorado.ceres[Dorado.ceres_keys[cr]].filters[phi]]
        
        if (fildat.target == None):
            fplate = str(int(Dorado.ceres[Dorado.ceres_keys[cr]].date.mjd)) + '-' + phi + '_'
        else:
            fplate = str(fildat.target.name) + '-' + str(int(Dorado.ceres[Dorado.ceres_keys[cr]].date.mjd)) + '-' + phi + '_' 
            
        if save_series:
            if (fildat.calibrated == True) and (fildat.aligned == True): 
                wrdir =     wrkdir / datestr / 'aligned'
                os.makedirs(wrkdir / datestr / 'aligned'      / phi, exist_ok = True)
                wrdir =     wrdir  / phi
                fsub = '_ca'
            elif (fildat.aligned == True):
                wrdir =     wrkdir / datestr / 'aligned'
                os.makedirs(wrkdir / datestr / 'aligned'      / phi, exist_ok = True)
                wrdir =     wrdir  / phi
                fsub = '_a'
            elif (fildat.calibrated == True):
                wrdir =     wrkdir / datestr / 'calibrated'
                os.makedirs(wrkdir / datestr / 'calibrated'   / phi, exist_ok = True)
                wrdir =     wrdir  / phi
                fsub = '_c'
            else: 
                wrdir =     wrkdir / datestr / 'uncalibrated'
                os.makedirs(wrkdir / datestr / 'uncalibrated' / phi, exist_ok = True)
                wrdir =     wrdir  / phi
                fsub = ''
                
            for p in range(len(fildat.data)):
                image             = fildat.data[p]
                tstr              = Time(image.header['DATE-OBS'], format='fits', 
                                        out_subfmt='date_hms').strftime('%Y_%m_%d-%H_%M_%S%z')
                image.data        = image.data.astype('uint16') 
                # image.mask = image.mask.astype('uint16') 
                image.mask        = None
                image.uncertainty = None
                # image.uncertainty = image.uncertainty.astype('uint16') 
                fname = fplate + '_' + tstr + fsub + '.fits'
                image.write(wrdir / fname, overwrite = True)
                
        if save_base:
            if fildat.base != None:
                fname = fplate + '_base.fits'
                fildat.base.write(wrkdir / datestr / fname, overwrite = True)
                
            if fildat.solved != None: 
                fname = fplate + '_solved.fits'
                fildat.solved.write(wrkdir / datestr / fname, overwrite = True)

    def getBias(self, mjdstr):
        # NOTE This function does not account for binning/image size
        target_mjd = int(mjdstr)
        biasdir = Dorado.dordir / 'data' / 'bias' 
        contents = os.scandir(path = biasdir)
        candidates = []
        # print(target_mjd)
        for entry in contents:
            entry_mjd = int(entry.name[0:5]) # grab first five characters of filename. Should be the mjd number
            # print(entry_mjd)
            if np.abs(target_mjd - entry_mjd) <= self.calibration_window:
                candidates.append(entry_mjd)
        if len(candidates) == 0:
            print('No viable bias frames found. Try increasing your calibration window.')
            bias = None
        else:
            mjd_to_use = candidates[np.argmin(np.array(candidates) - entry_mjd)] #locate the mjd of the closest file in time
            print('Retrieved bias from mjd: ', mjd_to_use)
            fname = str(mjd_to_use) + '_Bias.fits'
            bias = CCDDatax.read(biasdir / fname) #, unit = Dorado.unit) ## NOTE edited
        return bias

    def getFlat(self, mjdstr, phi):
        target_mjd = int(mjdstr)
        flatdir = Dorado.dordir / 'data' / 'flats' 
        contents = os.scandir(path = flatdir)
        candidates = []
        # print(target_mjd)
        for entry in contents:
            if phi in entry.name:
                entry_mjd = int(entry.name[0:5]) # grab first five characters of filename. Should be the mjd number
                # print(entry_mjd)
                if np.abs(target_mjd - entry_mjd) <= self.calibration_window:
                    candidates.append(entry_mjd)
        if len(candidates) == 0:
            print('No viable flat frames found for ', phi, '. Try increasing your calibration window or using a different filter.')
            flat = None
        else:
            mjd_to_use = candidates[np.argmin(np.array(candidates) - entry_mjd)]
            print('Retrieving ', phi, ' flat from mjd: ', mjd_to_use)
            fname = str(mjd_to_use) + '_' + str(phi) + '_flat.fits'
            flat = CCDDatax.read(flatdir / fname) #, unit = Dorado.unit) ## NOTE edited
        return flat

# class tess_reader:
#     def __init__(self):
#         Dorado.init_dir(tess = True)
#         # check if a name query is needed to retrieve data
#         # initialize 

#     def mkceres(self, tid, sub = 'raw', target = None):
#         fits_file = self.dirscan()
#         tpf = lk.open(fits_file)

#         with fits.open(fits_file, mode="readonly") as hdulist:
#             tess_bjds = hdulist[1].data['TIME'] # Should I use 'TIMECORR'?
#             tess_bjds_corr = hdulist[1].data['TIMECORR'] 
#             raw_counts = hdulist[1].data['RAW_CNTS']
#             calibrated_fluxes = hdulist[1].data['FLUX'] # Should I use raw or calibrated?
#             flux_err = hdulist[1].data['FLUX_ERR']
#             tid = hdulist[0].header['TICID']
#             TSTART = hdulist[0].header['TSTART']
#             TSTOP = hdulist[0].header['TSTOP']
#             tidstr = 'TIC ' + str(tid)

#         cere = Ceres()
#         # need to make a stack, and reconfigure the ceres class to be more generic in data storage. maybe make the dorphot function
#         # a wrapper, or make the ability to produce a target pixel file of aico data.
#         return cere


#     def dirscan(self, Dorado, dirarray):
#         path = Dorado.dordir
#         for dir in dirarray:
#             path = path / dir

#         files, directories = Dorado.diread(path)

#         if len(files) == 0:
#             print('No files found...')
#         elif len(files) > 1:
#             for file in files:
#                 if '_tp.fits' in file:
#                     tp_file = file    
#         elif len(files) == 1:
#             tp_file = files[0]

#         return tp_file




def _single_level(self, files):
    bias    = self._read_stack(files, 'bias')
    flats  = self._read_stack(files, 'flats')
    lights = self._read_stack(files, 'lights')
    return bias, flats, lights
    
def _multi_level_top(self, files, directories):
    biasdir = [s for s in directories if s.name in self.biasstr]
    # There should never be multiple bias directories; unless bias were taken with multiple binning settings or taken multiple times.
    # Consult chief data officer about this. TODO
    bias_files, _ = self.diread(biasdir[0])
    bias    = self._read_stack(bias_files, 'bias')
    # It is possible to have multiple flats directories for different filters, they should all be contained in a single base flats directory tho
    flatsdir = [s for s in directories if s.name in self.flatsstr]
    flats = self._multi_level_sub(flatsdir[0], 'flats')
    if self.unique_lights:
        lightsdir = [s for s in directories if (s.name not in self.flatsstr) and (s.name not in self.biasstr)] # may hand back more than one directory
    else:
        lightsdir = [s for s in directories if s.name in self.lightsstr]  # may hand back more than one directory
    lights = self.multi_level_sub(lightsdir[0], 'lights')
    return bias, flats, lights

def _multi_level_sub(self, subdir, stack_type):
    files, directories = self.diread(subdir)
    sub_files = []
    if len(directories) == 0:
        print('Single directory ', stack_type, ' organization format detected.')
        sub_files  = self._read_stack(files, stack_type)
    elif len(files) == 0:
        print('Multi directory ', stack_type, ' organization format detected.')
        for d in directories:
            # should not be any further sub folders
            f, _ = self.diread(d)
            sub_files += _read_stack(f, stack_type) # should we continue to hand stack_type if only the desired frame type should be present this deep in the mix?
    return sub_files
        
    

def _chkPhi(self, stack_files):
    # sequence through headers saving every filter keyword
    phi_list = []
    phi_stacks = {}
    for im in stack_files:
        phi = im.header['FILTER'] # hardcoded filter keyword for now
        if phi in phi_list: # check if filter is already accounted for 
            phi_stacks[phi].append(im)
        else:
            phi_stacks[phi] = [im]
    for p in phi_list:
        print('Found ', len(phi_stacks[p]), ' ', p, ' frames.')
    
    return phi_list, phi_stacks

def chkCal(self, frames):
    
    pass

def _read_stack(self, files, stack_type):
    if (stack_type == 'lights') & (self.unique_lights):
        stackstr = self.stack_types['bias'] + self.stack_types['flats'] # NOTE:: in the future this should be modded to include all non light stack types.
    else:
        try:
            stackstr = self.stack_types[stack_type]
        except:
            stackstr = stack_type
        
    stack_list = []
    for st in stackstr:
        for s in files:
            if (stack_type == 'lights') & (self.unique_lights):
                if (str(st)) not in str(s.name):
                    stack_list.append(s)
            else:
                if (str(st)) in str(s.name):
                    stack_list.append(s)
    stack_files = []
    if len(stack_list) > 50:
        n = 0 # this is a very janky way of writing this, enjoy
        for f in (pbar := tqdm(stack_list, colour = 'green')):
            n += 1
            pbar.set_description('Reading ' + stack_type + ' : ' + str(n))
            pbar.refresh()
            hdu = CCDData.read(f.path)
            stack_files.append(hdu)
    else:
        for f in stack_list:
            hdu = CCDData.read(f.path)
            stack_files.append(hdu)
    return stack_files

def getBias(mjdstr):
    target_mjd = int(mjdstr)
    biasdir = Dorado.dordir / 'data' / 'bias' 
    contents = os.scandir(path = biasdir)
    candidates = []
    for entry in contents:
        entry_mjd = int(entry.name[0:4]) # grab first five characters of filename. Should be the mjd number
        if np.abs(target_mjd - entry_mjd) <= self.calibration_window:
            candidates.append(entry_mjd)
    if len(candidates) == 0:
        print('No viable bias frames found. Try increasing your calibration window.')
    else:
        mjd_to_use = candidates[np.argmin(candidates- entry_mjd)]
    
    fname = str(mjd_to_use) + '_Bias.fits'
    bias = CCDData.read(biasdir / fname) #, unit = Dorado.unit) ## NOTE edited
    return bias

def getFlat(self, mjdstr, phi):
    target_mjd = int(mjdstr)
    flatdir = Dorado.dordir / 'data' / 'flats' 
    contents = os.scandir(path = flatdir)
    candidates = []
    for entry in contents:
        if phi in entry.name:
            entry_mjd = int(entry.name[0:4]) # grab first five characters of filename. Should be the mjd number
            if np.abs(target_mjd - entry_mjd) <= self.calibration_window:
                candidates.append(entry_mjd)
    if len(candidates) == 0:
        print('No viable flat frames found. Try increasing your calibration window or using a different filter.')
    else:
        mjd_to_use = candidates[np.argmin(candidates- entry_mjd)]
    
    fname = str(mjd_to_use) + '_' + str(phi) + '_flat.fits'
    flat = CCDData.read(flatdir / fname) #, unit = Dorado.unit) ## NOTE edited
    return flat


import warnings
warnings.filterwarnings('ignore')

from ..ceres import Ceres
from ..stack import Stack
from ..core.readerClass import *
from ..dorphot.dorphotClass import *
from ..target.targetClass import *
from ..core.cacheClass import *

import ccdprocx

from astropy.nddata.ccddata import CCDData
CCDData._config_ccd_requires_unit = False
import numpy as np
from astropy import config as _config
from astropy.utils.misc import isiterable
import astropy.units as un
from astropy.time import Time


from astroquery.astrometry_net import AstrometryNet
from astroquery.exceptions  import TimeoutError
from astropy.wcs import WCS

import os
import datetime
from pathlib import Path


ast = AstrometryNet()

'''
Filer is the handler of the Dorado system,
'''

__all__ = ['Dorado', 'Dorado_core']


def get_builtins():
    """Due to the way Python works, ``__builtins__`` can strangely be either a module or a dictionary,
    depending on whether the file is executed directly or as an import. I couldn’t care less about this
    detail, so here is a method that simply returns the namespace as a dictionary."""
    return getattr( __builtins__, '__dict__', __builtins__ )


class Dorado_core:
    '''
    
    '''
    def __init__(self):
        # open and use logger
        # make function to create data class from hardware, processed, or raw data folder
        # needs function to import data into raw
        # fix get_night()
        # add new calibration frames to folders
        # find most recent calibration frame if none
        # -> figure out if none
        # clear cache function
        self.stardir = os.getcwd()
        self.rootname = 'dorado'
        self.config_dir = _config.get_config_dir(self.rootname)
        # read in config from config_dir
        
        self.dordir = Path(self.config_dir).parent
        self.init_dir()
        self.unit = un.adu

        ## class storage zone -----------------------
        # redo this so theres an easy storage and readout
        # reader
        self.reader = aico_reader() # ussually we'd want to grab this from a configuration file, should name be aicoReader?
        self.cache = cache_core()   # add cache handler to Dorado core instance
        # dorphot
        self.dorphot = aicoPhot()
        # ceres & stacks
        self.ceres_keys = {}
        self.ceres = [] # maybe call this series?
        # self.filters[stack.filter] = len(self.data)
        # self.data.append(stack)

        # targets
        self.target_keys = {}
        self.targets = [] # TODO choose target class
        # all early ts instances will belong to a target;
        # timeseries
        self.timeseries = [] # exclude this for the time being as 

        ### read from config
        # site information (hardcoded atm)
        self.UTCoffset = -5

    def target(self, key):
        return self.targets[self.target_keys[key]]
    
    def cere(self, key):
        return self.ceres[self.ceres_keys[key]]
    
    def init_dir(self, tess = False):
        """
        init_dir initializes the dorado home directory '$user/.dorado/'.
        
        Parameters
        ----------
        tess: boolean
            Whether to initialize a TESS data directory in './data/'
        Returns
        -------
        
        """
        self.enter_dordir()
        os.makedirs('./data/wrk',            exist_ok = True)
        os.makedirs('./data/flats',          exist_ok = True)
        os.makedirs('./data/bias',           exist_ok = True)
        os.makedirs('./data/darks',          exist_ok = True)
        os.makedirs('./data/raw',            exist_ok = True)
        os.makedirs('./data/graphical',      exist_ok = True)
        os.makedirs('./data/projects',       exist_ok = True)
        os.makedirs('./data/targets',        exist_ok = True)
        os.makedirs('./logs',                exist_ok = True)
        os.makedirs('./cache',               exist_ok = True)
        os.makedirs('./cache/astrometryNet', exist_ok = True)
        if tess:
            os.makedirs('./data/tess',       exist_ok = True)
        self.exit_dordir()
        
    def enter_dordir(self):
        """
        enter_dordir changes the workin directory to self.dordir which by default is
        '$user/.dorado/'.
        
        Parameters
        ----------

        Returns
        -------
        
        """
        os.chdir(self.dordir)

    def exit_dordir(self):
        """
        exit_dordir changes the workin directory to the starting directory at runtime.
        
        Parameters
        ----------

        Returns
        -------
        
        """
        os.chdir(self.stardir)
        
    def mkceres(self, date, name = None, sub = 'raw', target = None, calibrated = False, aligned = False):
        """
        mkceres is a wrapper function for self.reader.mkceres with an additional arguement for setting the keyname 
        for the ceres instance in self.ceres_keys. The default keyname is the datestring for the ceres instance.
        
        Parameters
        ----------
        name: string
            keyname for the ceres instance. Default is the datestring for the ceres instance.
        
        self.reader.mkceres args** 
        
        Returns
        -------
        
        """
        if target == None:
            ctemp = self.reader.mkceres(date, sub = sub, target = target, calibrated = calibrated, aligned = aligned)
        else:
            ctemp = self.reader.mkceres(date, sub = sub, target = self.targets[self.target_keys[target]], calibrated = calibrated, aligned = aligned)
        # add logic to print out a summary

        if name == None:
            name = ctemp.datestr
            # TODO handle if no date found either
            print('No series nickname given, defaulting to date string: ', name)
        else:
            print('Call series as: ', name, '\n')
        self.ceres_keys[name] = len(self.ceres)
        self.ceres.append(ctemp)
    
    def mktrgt(self, name, constructor = Target):
        """
        mktrgt takes a target name and constructs a dorado.target object that is returned to self.targets.
        
        Parameters
        ----------
        name: string
            target name as found in SIMBAD
        Returns
        -------
        
        """
        # TODO allow for nickname in keys
        # add dictionary
        # , coordinates = None
        self.target_keys[name] = len(self.targets)
        self.targets.append(constructor(name))
        
    def savewrk(self, cr, filters = None, save_series = True, save_base = True):
        """
        savewrk is a wrapper function for self.reader.savewrk.
        Parameters
        ----------

        self.reader.savewrk args**
        Returns
        -------
        
        """
        self.reader.savewrk(cr, filters, save_series, save_base)

    # logic to set reader and dorphot
        
        
        
    # move to ceres
    def getDateString(self, cr, utc = None):
        """
        getDateString computes the date string for the given ceres object. ceres time is represented
        in UTC while datestring is represented in local time. A UTC offset must be provided either by self.UTCoffest
        or by the utc arguement.
        
        Parameters
        ----------
        cr : str
            the ceres object key to get the date string for.
        utc: float
            utc offset to apply to the date string. Default is None.

        Returns
        -------
        
        """

        ## TODO :: look into UTC wrecking stuff
        # if the hour is less than the utc offset of the site then the utc date is one ahead of local time
        epoch = self.ceres[self.ceres_keys[cr]].date.ymdhms

        # epoch = date.ymdhms
        if (utc):
            
            if (epoch['hour'] + utc) < 0:
                day = str(epoch['day'] - 1)
                day2 = str(epoch['day'])
                month = str(epoch['month'])

                if (epoch['day'] - 1) < 10:
                    day = '0' + str(epoch['day'] - 1)
                if epoch['day'] < 10:
                    day2 = '0' + str(epoch['day'])

                if epoch['month'] < 10:
                    month = '0' + str(epoch['month'])

            else: 
                day = str(epoch['day'])
                day2 = str(epoch['day'] + 1)
                month = str(epoch['month'])

                if epoch['day'] < 10:
                    day = '0' + str(epoch['day'])
                    if epoch['day'] < 9:
                        day2 = '0' + str(epoch['day'] + 1)

                if epoch['month'] < 10:
                    month = '0' + str(epoch['month'])
        else:
            if (epoch['hour'] + self.UTCoffset) < 0:
                day = str(epoch['day'] - 1)
                day2 = str(epoch['day'])
                month = str(epoch['month'])

                if (epoch['day'] - 1) < 10:
                    day = '0' + str(epoch['day'] - 1)
                if epoch['day'] < 10:
                    day2 = '0' + str(epoch['day'])

                if epoch['month'] < 10:
                    month = '0' + str(epoch['month'])

            else: 
                day = str(epoch['day'])
                day2 = str(epoch['day'] + 1)
                month = str(epoch['month'])

                if epoch['day'] < 10:
                    day = '0' + str(epoch['day'])
                    if epoch['day'] < 9:
                        day2 = '0' + str(epoch['day'] + 1)

                if epoch['month'] < 10:
                    month = '0' + str(epoch['month'])


        datestr = str(epoch['year']) + '-' + month + '-' + day + '+' + day2
        self.ceres[self.ceres_keys[cr]].datestr = datestr
        
    # move to reader
    def mkwrk(self, cr):
        """
        mkwrk produces a working folder for saving a ceres object to the dorado data working directory
        for later use.
        
        Parameters
        ----------
        cr : str
            ceres key for ceres instance to create working directory for.
        Returns
        -------
        
        """
        # TODO this should be a wrapper for reader
        if self.ceres[self.ceres_keys[cr]].datestr == None:
            self.getDateString(cr) 
        datestr = self.ceres[self.ceres_keys[cr]].datestr
        wrkdir = self.dordir / 'data' / 'wrk'
        os.makedirs(wrkdir / datestr,                  exist_ok = True)
        os.makedirs(wrkdir / datestr / 'aligned',      exist_ok = True)
        os.makedirs(wrkdir / datestr / 'calibrated',   exist_ok = True)
        os.makedirs(wrkdir / datestr / 'uncalibrated', exist_ok = True)
        os.makedirs(wrkdir / datestr / 'WCS',          exist_ok = True)
        # figures, targets, log, omitted images, observation metadata

    # move to utils or reader
    def saveWCS(self, cr, filters = None):
        """
        saveWCS takes a ceres object and an optional filters argument and saves the contained WCS
        data to the ceres objects working directory.
        
        Parameters
        ----------
        cr : str
            ceres object key string for the relevent ceres object.
        filters : str or list of str
            filters of WCS data to be saved.

        Returns
        -------
        
        """
        wrkdir = self.dordir / 'data' / 'wrk'
        if self.ceres[self.ceres_keys[cr]].datestr == None:
            self.getDateString(cr)
        datestr = self.ceres[self.ceres_keys[cr]].datestr
        self.mkwrk(cr)
        if filters == None:
            filters = self.ceres[self.ceres_keys[cr]].filters.keys()
        
        for filter in filters:
            stack = self.ceres[self.ceres_keys[cr]].data[self.ceres[self.ceres_keys[cr]].filters[filter]]
            if stack.wcs == None:
                self.dorphot.getWCS(filter, self) # TODO this is now in dorphot, should it be moved?
            fname = str(filter) + '-solved.fits'
            solved = self.ceres[self.ceres_keys[cr]].data[self.ceres[self.ceres_keys[cr]].filters[filter]].solved
            solved.data = solved.data.astype('uint16')
            solved.mask = None
            solved.uncertainty = None
            solved.write(wrkdir / datestr / 'WCS' / fname, overwrite = True)
        # not done might be redundant atm also shouldnt this belong to reader?
    
    # move to ceres or reader
    def saveBase(self, cr, filters = None):
        """
        saveBase save the base frame of a ceres object if any.
        Parameters
        ----------
        cr: string
            ceres key string for relevent ceres object.
        filters: str or list of str
            filters to save base frame of a ceres object.
        Returns
        -------
        
        """
        wrkdir = self.dordir / 'data' / 'wrk'
        if self.ceres[self.ceres_keys[cr]].datestr == None:
            self.getDateString(cr)
        datestr = self.ceres[self.ceres_keys[cr]].datestr

        print('Saved to data/wrk/', datestr)
        self.mkwrk(cr)

        if filters == None:
            filters = self.ceres[self.ceres_keys[cr]].filters.keys()
        
        for filter in filters:
            imname = filter + '_base.fits'
            fname = wrkdir / datestr / imname
            base = self.ceres[self.ceres_keys[cr]].data[self.ceres[self.ceres_keys[cr]].filters[filter]].base
            base.data        = base.data.astype('uint16') 
            # base.mask        = base.mask.astype('uint16')
            # base.uncertainty = base.uncertainty.astype('uint16')
            base.mask = None
            base.uncertainty = None
            base.write(fname, overwrite = True)
        # not done might be redundant atm also shouldnt this belong to reader?
        
        # merge header

Dorado = Dorado_core()

# this gone pollute the namespace but ay lmao
G = get_builtins()
G[ 'G' ] = G
G['Dorado'] = Dorado



# we need to pass the calibration files(if none grab the most recent out of 
# dorado by closest mjd)
# if there is no calibration files in dorado send a warning and set 
# calibration files to none
# handle multi filter




import os

__all__ = ['cache_core']

class cache_core:
    '''
    '''
    def __init__(self):
        pass

    def mkcacheObj(self, object, subcache = False):
        """
        mkcacheObj is a convenience function that creates a cache object for a .fits compatable
        object in the self.dordir/cache directory.
        
        Parameters
        ----------
        object: CCDData
            object to be saved in the cache directory for access.
        subcache: str
            optional subcache file for cache organization. Optional
        Returns
        -------
        
        """
        if subcache:
            cachedir = Dorado.dordir / 'cache' / subcache
            dirarray = ['cache', subcache]
        else:
            cachedir = Dorado.dordir / 'cache' 
            dirarray = ['cache']
        # if isiterable(object):
        #     # print('This function does not currently support iterable objects.')
        #     return warnings.WarningMessage('This function does not currently support iterable objects.')
        # else:
        #     files, _ = self.diread(dirarray)
        #     fname = 'cache_object_' + str(len(files) + 1) + '.fits'
        #     object.write(fname)
        #     return(fname, cachedir)

        # check if iterable
        # NOTE moved to reader
        files, _ = Dorado.reader.diread(dirarray)
        # TODO will the cache object always be a fits file? what about JSON?
        fname = 'cache_object_' + str(len(files) + 1) + '.fits'
        # using object name seems fishy, is it a protected word?
        object.write(cachedir / fname, overwrite = True)
        return(fname, cachedir)
        
    def delcacheObj(self, fname, subcache = False):
        """
        delcacheObj is a convenience method that deletes a selected cache object given is file name.
        
        Parameters
        ----------
        fname: string
            filestring of cache object to delete.
        subcache: string
            filestring of subcache containing the cache object to delete.
        Returns
        -------
        
        """
        # TODO extend to clearing cache by accepting None type as fname and reading the dir for all files
        if subcache:
            cachedir = Dorado.dordir / 'cache' / subcache
        else:
            cachedir = Dorado.dordir / 'cache'

        os.remove(cachedir / fname)
        


#!/usr/bin/env python
# Licensed under a 3-clause BSD style license - see LICENSE.rst

# NOTE: The configuration for the package, including the name, version, and
# other information are set in the setup.cfg file.

import os
import sys

from setuptools import setup

# To check version before setup, run:
# python setup.py --version

# To push to PyPi:
# python setup.py sdist bdist_wheel --universal
# twine upload dist/*  
# python -m twine upload dist/*  

# First provide helpful messages if contributors try and run legacy commands
# for tests or docs.

TEST_HELP = """
Note: running tests is no longer done using 'python setup.py test'. Instead
you will need to run:

    tox -e test

If you don't already have tox installed, you can install it with:

    pip install tox

If you only want to run part of the test suite, you can also use pytest
directly with::

    pip install -e .[test]
    pytest

For more information, see:

    http://docs.astropy.org/en/latest/development/testguide.html#running-tests
"""

if 'test' in sys.argv:
    print(TEST_HELP)
    sys.exit(1)

DOCS_HELP = """
Note: building the documentation is no longer done using
'python setup.py build_docs'. Instead you will need to run:

    tox -e build_docs

If you don't already have tox installed, you can install it with:

    pip install tox

You can also build the documentation with Sphinx directly using::

    pip install -e .[docs]
    cd docs
    make html

For more information, see:

    http://docs.astropy.org/en/latest/install.html#builddocs
"""

if 'build_docs' in sys.argv or 'build_sphinx' in sys.argv:
    print(DOCS_HELP)
    sys.exit(1)

VERSION_TEMPLATE = """
# Note that we need to fall back to the hard-coded version if either
# setuptools_scm can't be imported or setuptools_scm can't determine the
# version, so we catch the generic 'Exception'.
try:
    from setuptools_scm import get_version
    version = get_version(root='..', relative_to=__file__)
except Exception:
    version = '{version}'
""".lstrip()

setup(
    version='2.0.0.dev66'
    # use_scm_version={
    #     'write_to': os.path.join('dorado', 'version.py'),
    #     'write_to_template': VERSION_TEMPLATE,
    #     "local_scheme": "no-local-version",
    #     "version_scheme": "python-simplified-semver"
    #                    }
        )
# where is this file? os.path.dirname(__file__)
# import  astropy.config as acfg
# import ./dorado/config


# Entry points and automatic script creation
# Setuptools supports automatic creation of scripts upon installation, that run code within your package if you specify them as entry points. An example of how this feature can be used in pip: it allows you to run commands like pip install instead of having to type python -m pip install.

# https://setuptools.pypa.io/en/latest/userguide/quickstart.html









# # Licensed under a 3-clause BSD style license - see LICENSE.rst

# __all__ = ['__version__']

# # this indicates whether or not we are in the package's setup.py
# try:
#     _ASTROPY_SETUP_
# except NameError:
#     import builtins
#     builtins._ASTROPY_SETUP_ = False

# try:
#     from .version import version as __version__
# except ImportError:
#     __version__ = ''


# if not _ASTROPY_SETUP_:  # noqa
#     import os
#     from warnings import warn
#     from astropy.config.configuration import (
#         update_default_config,
#         ConfigurationDefaultMissingError,
#         ConfigurationDefaultMissingWarning)

#     # Create the test function for self test
#     from astropy.tests.runner import TestRunner
#     test = TestRunner.make_test_runner_in(os.path.dirname(__file__))
#     test.__test__ = False
#     __all__ += ['test']

#     # add these here so we only need to cleanup the namespace at the end
#     config_dir = None

#     if not os.environ.get('ASTROPY_SKIP_CONFIG_UPDATE', False):
#         config_dir = os.path.dirname(__file__)
#         config_template = os.path.join(config_dir, __package__ + ".cfg")
#         if os.path.isfile(config_template):
#             try:
#                 update_default_config(
#                     __package__, config_dir, version=__version__)
#             except TypeError as orig_error:
#                 try:
#                     update_default_config(__package__, config_dir)
#                 except ConfigurationDefaultMissingError as e:
#                     wmsg = (e.args[0] +
#                             " Cannot install default profile. If you are "
#                             "importing from source, this is expected.")
#                     warn(ConfigurationDefaultMissingWarning(wmsg))
#                     del e
#                 except Exception:
#                     raise orig_error

# Licensed under a 3-clause BSD style license - see LICENSE.rst

__all__ = ['__version__']

# this indicates whether or not we are in the package's setup.py
try:
    _ASTROPY_SETUP_
except NameError:
    from sys import version_info
    if version_info[0] >= 3:
        import builtins
    else:
        import __builtin__ as builtins
    builtins._ASTROPY_SETUP_ = False

try:
    from .version import version as __version__
except ImportError:
    __version__ = ''
